module('entity.lightning', package.seeall)

local random = math.random

function init(e)
	scriptEntities[e] = {
		e = e,
		intensity = 0
	}
	amb = Arkham.getAmbientLight()
end

soundQueue = {}

local function enqueueSound(name, time)
	table.insert(soundQueue, {name=name, time=time})
end

local function processSoundQueue()
	local now = Arkham.getWorldTime()
	for i, snd in pairs(soundQueue) do
		if now >= snd.time then
			Arkham.Audio.playSound(snd.name)
			table.remove(soundQueue, i)
		end
	end
end


local function thunderSound(intensity)
	local sound
	if intensity + 0.5 * (math.random() - 0.5) > 0.5 then
		sound = 'sound/heavythunder.ogg'
	else
		sound = 'sound/lightthunder' .. math.random(2) .. '.ogg'
	end
	enqueueSound(sound, Arkham.getWorldTime() + 2 - intensity + math.random() - 0.5)
end

local function lightning(intensity)
	local brightness = 300 + intensity * random() * 700
	local lightningAmb = {brightness, brightness, brightness}

	local pause = (5 + random() * 3) * (1 - 0.75 * intensity)
	local duration = (0.2 + random() * 0.5) * (0.5 + 0.5 * intensity)
	local double = (random() <= 0.75 * intensity)
	
	wait(pause)
	Arkham.setAmbientLight(lightningAmb)
	thunderSound(intensity)
	wait(duration)
	if double then
		Arkham.setAmbientLight(amb)
		wait(duration/2)
		Arkham.setAmbientLight(lightningAmb)
		wait(duration/2)
	end
	Arkham.setAmbientLight(amb)
end

function think(e)
	local data = scriptEntities[e]
	yield()
	while true do
		local msg = receiveMessage(e)
		while msg ~= nil do
			if msg.name == 'setIntensity' then
				data.intensity = msg.intensity
			end
			msg = receiveMessage(e)
		end
	
		lightning(data.intensity)
	end
end

function update(e, dt)
	processSoundQueue()
end
