--
-- util.lua
--
-- Utility functions.
--


-- short cuts
_e = Arkham.Entity.getEntityByName
yield = coroutine.yield


function createEntityAt(pos)
	local e = Arkham.Entity.create()
	Arkham.Entity.createComponent(e, 'location')
	Arkham.Entity.setPosition(e, pos)
	return e
end


function setupLight(e, radius, falloff, color)
	Arkham.Entity.createComponent(e, 'light')
	Arkham.Entity.setLightRadius(e, radius)
	Arkham.Entity.setLightFalloff(e, falloff)
	Arkham.Entity.setLightColor(e, color)
end


function table.copy(t)
	local c = {}
	for key, val in pairs(t) do
		c[key] = val
	end
	return c
end

Direction =
{
	North = 0,
	NorthEast = 1,
	East = 2,
	SouthEast = 3,
	South = 4,
	SouthWest = 5,
	West = 6,
	NorthWest = 7,
}


function string.startsWith(str, prefix)
	return str:sub(1, prefix:len()) == prefix
end

function sendMessage(sender, recv, name, param)
	if type(param) ~= 'table' then
		param = {}
	end
	param.sender = sender
	param.name = name
	
	if scriptEntities[recv]._messages == nil then
		scriptEntities[recv]._messages = { param }
	else
		table.insert(scriptEntities[recv]._messages, param)
	end
end

function receiveMessage(recv)
	local queue = scriptEntities[recv]._messages
	if queue == nil or #queue == 0 then
		return nil
	end
	
	-- return first entry in message queue and remove it
	return table.foreachi(queue, function(i, msg)
		table.remove(queue, i)
		return msg
	end)
end


function isEntityAdjacent(e, p)
	return GridPos.isAdjacent(Arkham.Entity.getPosition(e), p)
end


function wait(time)
	local endTime = Arkham.getWorldTime() + time
	while Arkham.getWorldTime() < endTime do
		yield()
	end
end


function dumpTable(t)
	for i, v in pairs(t) do
		print(i, v)
	end
end

function textBox(text)
	Arkham.textBox({-1,-1}, 80, text)
end


function fadeOut(duration)
	local original = Arkham.getAmbientLight()
	local now = Arkham.getWorldTime
	local start = now()
	
	while now() - start <= duration do
		local fade = math.max(0, 1 - (now() - start) / duration)
		Arkham.setAmbientLight({original[1] * fade, original[2] * fade, original[3] * fade})
		yield()
	end
	Arkham.setAmbientLight({0, 0, 0})
end
